﻿using System;
using System.Collections;
using System.Collections.Specialized;
using System.Collections.Generic;
using System.Text;
using System.Web;
using System.Net;
using System.Web.UI;
using System.Web.UI.WebControls;

using CMS.SiteProvider;
using CMS.Ecommerce;
using CMS.GlobalHelper;
using CMS.SettingsProvider;
using CMS.CMSHelper;
using CMS.EventLog;
using CMS.EcommerceProvider;

/// <summary>
/// Class providing payment using WordPay payment gateway.
/// </summary>
public class WorldPayProvider : CMSPaymentGatewayProvider
{
    /// <summary>
    /// Creates WorldPay payment result object.
    /// </summary>
    public override PaymentResultInfo CreatePaymentResultInfo()
    {
        return new WorldPayPaymentResultInfo();
    }


    /// <summary>
    /// Adds payment gateway custom data to the specified shopping cart step.
    /// </summary>        
    public override CMSPaymentGatewayForm GetPaymentDataForm()
    {
        try
        {
            Control ctrl = ShoppingCartControl.LoadControl("~/CMSModules/Ecommerce/Controls/PaymentGateways/WorldPayForm.ascx");
            ctrl.ID = "shoppingCartControl";
            return (CMSPaymentGatewayForm)ctrl;
        }
        catch (Exception ex)
        {
            try
            {
                int siteId = 0;
                int userId = 0;
                string userName = "";


                // Get current user information
                UserInfo ui = CMSContext.CurrentUser;
                if (ui != null)
                {
                    userId = ui.UserID;
                    userName = ui.UserName;
                }

                // Get current site information
                SiteInfo si = CMSContext.CurrentSite;
                if (si != null)
                {
                    siteId = si.SiteID;
                }

                // Log the event
                EventLogProvider evenLog = new EventLogProvider();
                evenLog.LogEvent(EventLogProvider.EVENT_TYPE_ERROR, DateTime.Now, "WorldPayProvider", "EXCEPTION", userId, userName, 0, "", "", EventLogProvider.GetExceptionLogMessage(ex), siteId, URLHelper.CurrentURL);
            }
            catch
            {
                // Unable to log the event
            }
        }
        return null;
    }


    #region "Payment"

    /// <summary>
    /// Process payment - initialize payment request and redirect to the WorldPay payment gateway to finish payment.
    /// </summary>
    public override void ProcessPayment()
    {

        if (ShoppingCartInfoObj != null)
        {
            // Get payment gateway url
            string url = GetPaymentGatewayUrl();
            if (url != "")
            {
                Hashtable requestParams = new Hashtable();

                requestParams["lang"] = ShoppingCartInfoObj.ShoppingCartCulture;
                // Custom parameter to return back in PM message
                requestParams["M_culture"] = ShoppingCartInfoObj.ShoppingCartCulture;

                // Initialize request
                ErrorMessage = InitializePaymentRequest(requestParams);

                if (ErrorMessage == "")
                {
                    // Remove shopping cart from database and session
                    ShoppingCartControl.CleanUpShoppingCart();

                    try
                    {
                        // Redirect to the WorldPay (alternative approach)
                        //this.ShoppingCartControl.Page.URLHelper.Redirect(GetFullPaymentGatewayUrl(url, requestParams));                            

                        ShoppingCartControl.ButtonNext.Enabled = false;

                        // Redirect to WorldPay using javascript 
                        url = URLHelper.ResolveUrl(url);
                        ScriptHelper.RegisterClientScriptBlock(ShoppingCartControl.Page, typeof(string), "RedirectScript", ScriptHelper.GetRedirectScript(GetFullPaymentGatewayUrl(url, requestParams), true));

                    }
                    catch
                    {
                        // Error connecting to the payment gateway
                        ErrorMessage = ResHelper.GetString("PaymentGatewayProvider.ConnectionFailed");
                        ShoppingCartControl.ButtonNext.Enabled = true;
                    }
                }
                else
                {
                    // Error while initializing payment request
                }
            }
            else
            {
                // Payment gateway url not defined
                ErrorMessage = ResHelper.GetString("PaymentGatewayProvider.UrlNotDefined");
            }
        }
        else
        {
            // Order data not found
            ErrorMessage = ResHelper.GetString("PaymentGatewayProvider.paymentdatanotfound");
        }
    }


    /// <summary>
    /// Returns complete payment gateway url with initialized querystring.
    /// </summary>
    /// <param name="url">Base payment gateway url</param>
    /// <param name="requestParams">Name-Value collection with url parameters and their values</param>
    private string GetFullPaymentGatewayUrl(string url, Hashtable requestParams)
    {
        foreach (DictionaryEntry parameter in requestParams)
        {
            url = AddParameterToUrl(url, Convert.ToString(parameter.Key), HttpUtility.UrlEncode(Convert.ToString(parameter.Value)));
        }
        return url;
    }


    /// <summary>
    /// Adds parameter to specified URL.
    /// </summary>
    /// <param name="url">URL to be modified</param>
    /// <param name="parameterName">Parameter name</param>
    /// <param name="parameterValue">Parameter value</param>
    public static string AddParameterToUrl(string url, string parameterName, string parameterValue)
    {
        if (url == null)
        {
            return null;
        }

        // Get query
        string query = "";

        int queryIndex = url.IndexOf("?");
        if (queryIndex >= 0)
        {
            query = url.Substring(queryIndex);
            url = url.Substring(0, queryIndex);
        }

        // Add the parameter
        if (!String.IsNullOrEmpty(parameterName))
        {
            query = URLHelper.AddUrlParameter(query, parameterName, parameterValue);
        }

        return url + query;
    }


    /// <summary>
    /// Fills input name-value collection with payment request parameters' names and their values. Values are not URL-encoded.
    /// </summary>
    /// <param name="requestParams">Request parameters collection to fill</param>        
    /// <returns>Returns error message when some error occurs.</returns>
    public virtual string InitializePaymentRequest(Hashtable requestParams)
    {
        string errorMessage = "";

        if (ShoppingCartInfoObj != null)
        {
            string siteName = CMSContext.CurrentSiteName;

            // Installation ID (mandatory)
            requestParams["instId"] = SettingsKeyProvider.GetStringValue(siteName + ".CMSWorldPayInstallationID");

            // Unique ID (OrderId) (mandatory)
            requestParams["cartId"] = ShoppingCartInfoObj.OrderId;
            // Custom parameter to return back in PM message
            requestParams["M_orderId"] = ShoppingCartInfoObj.OrderId;
            
            // Currency (mandatory)
            if (ShoppingCartInfoObj.CurrencyInfoObj != null)
            {
                requestParams["currency"] = ShoppingCartInfoObj.CurrencyInfoObj.CurrencyCode;
            }

            // Total price - Two decimal places in number (mandatory)
            requestParams["amount"] = string.Format(CultureHelper.EnglishCulture.NumberFormat, "{0:0.##}", ShoppingCartInfoObj.RoundedTotalPrice);
           
            // Test mode
            if (ValidationHelper.GetBoolean(SettingsKeyProvider.GetBoolValue(siteName + ".CMSWorldPayTestMode"), false) == true)
            {
                requestParams["testMode"] = "100";
            }

            // Order description (optional)
            requestParams["desc"] = GetOrderDescription();
            
            // Customer's full name (optional)
            requestParams["name"] = ShoppingCartInfoObj.CustomerInfoObj.CustomerFirstName + " " + ShoppingCartInfoObj.CustomerInfoObj.CustomerLastName;

            // Disables currency selection
            requestParams["hideCurrency"] = "";
            
            // Customer's e-mail (optional)
            if (!string.IsNullOrEmpty(ShoppingCartInfoObj.CustomerInfoObj.CustomerEmail))
            {
                requestParams["email"] = ShoppingCartInfoObj.CustomerInfoObj.CustomerEmail;
            }
            
            // Billing address fixed data (optional)
            if (ValidationHelper.GetBoolean(SettingsKeyProvider.GetBoolValue(siteName + ".CMSWorldPayFixContact"), false) == true)
            {
                requestParams["fixContact"] = "";
            }

            // Hide the contact details (optional)
            if (ValidationHelper.GetBoolean(SettingsKeyProvider.GetBoolValue(siteName + ".CMSWorldPayHideContact"), false) == true)
            {
                requestParams["hideContact"] = "";
            }
            
            // Preferred merchant account
            string merchantAcount = ValidationHelper.GetString(SettingsKeyProvider.GetStringValue(siteName + ".CMSWorldPayMerchantAccount"), string.Empty);
            if (!string.IsNullOrEmpty(merchantAcount))
            {
                requestParams["accId1"] = merchantAcount;
                // Custom parameter to return back in PM message
                requestParams["M_accId1"] = merchantAcount;
            }

            // Extra billing information
            // town (e.g.: "my town")
            // region (e.g.: "my region or county")
            // postcode (e.g.: "AB10 5AB")
            // country (e.g.: "GB")
            // tel (e.g.: "0123456789")
            // address1 (e.g.: "4 The Street")
            // address2 (e.g.: "My Suburb")
            // address3 (e.g.: "")
        }

        return errorMessage;
    }


    /// <summary>
    /// Builds an order description from the order data which is attached to the payment request.
    /// </summary>
    public virtual string GetOrderDescription()
    {
        if (ShoppingCartInfoObj != null)
        {
            return string.Format(ResHelper.GetString("WorldPay.OrderDescription"), ShoppingCartInfoObj.OrderId);
        }
        return "";
    }

    #endregion


    #region "PN - Payment notification"

    /// <summary>
    /// Compares payment data (Seller account, total price, payment currency) to order data and returns appropriate error message if data don't match, otherwise returns empty string.
    /// </summary>
    /// <param name="collection">Name-Value collectin with the payment data that were posted to the merchant by WorldPay</param>
    public virtual string ComparePaymentDataToOrderData(NameValueCollection collection)
    {
        string errorMessage = "";

        if (ShoppingCartInfoObj != null)
        {

            // 1) Callback password validation

            // Get callback password
            string callbackPW = ValidationHelper.GetString(collection["callbackPW"], "");
            // Get Kentico callback passowrd from settings
            string callbackPWKentico = SettingsKeyProvider.GetStringValue(CMSContext.CurrentSiteName + ".CMSWorldPayCallbackPassword");

            //  callback password validation failed  -> error
            if (callbackPWKentico.Trim() != callbackPW)
            {
                errorMessage += string.Format(ResHelper.GetString("WorldPay.CallBackPasswordFailed"));
            }
            
            // 2) Total price validation

            double paymentTotalPrice = ValidationHelper.GetDouble(collection["authAmount"], 0);
            double orderTotalPrice = 0;

            // Get order total price
            OrderInfo oi = OrderInfoProvider.GetOrderInfo(ShoppingCartInfoObj.OrderId);
            if (oi != null)
            {
                orderTotalPrice = oi.OrderTotalPrice;
            }

            // Total price validation failed  -> error
            if (paymentTotalPrice != orderTotalPrice)
            {
                errorMessage += string.Format(ResHelper.GetString("WorldPay.OrderTotalPriceValidationFailed"), orderTotalPrice, paymentTotalPrice);
            }

            // 3) Currency validation

            string paymentCurrency = collection["authCurrency"];
            string orderCurrency = "";
            if (ShoppingCartInfoObj.CurrencyInfoObj != null)
            {
                orderCurrency = ShoppingCartInfoObj.CurrencyInfoObj.CurrencyCode;
            }

            // Curency validation failed  -> error
            if (paymentCurrency.ToLower() != orderCurrency.ToLower())
            {
                errorMessage += string.Format(ResHelper.GetString("WorldPay.CurrencyValidationFailed"), orderCurrency, paymentCurrency);
            }
            
            // 4) Seller account -> make sure that the transaction is for current merchant's account
        
            // Get merchant account
            string mAccount = collection["M_accId1"];

            // Get current merchant account 
            string merchantAcount = SettingsKeyProvider.GetStringValue(CMSContext.CurrentSiteName + ".CMSWorldPayMerchantAccount");

            // Seller validation failed -> error
            if (!string.IsNullOrEmpty(mAccount) && !string.IsNullOrEmpty(merchantAcount) && mAccount.ToLower() != merchantAcount.ToLower())
            {
                errorMessage += string.Format(ResHelper.GetString("WorldPay.SellerEmailValidationFailed"), mAccount, merchantAcount);
            }
        }

        return errorMessage;
    }

    #endregion
}

